'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _domLib = require('dom-lib');

var _shallowEqual = require('./utils/shallowEqual');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _decorate = require('./utils/decorate');

var _decorate2 = _interopRequireDefault(_decorate);

var _Row = require('./Row');

var _Row2 = _interopRequireDefault(_Row);

var _CellGroup = require('./CellGroup');

var _CellGroup2 = _interopRequireDefault(_CellGroup);

var _Scrollbar = require('./Scrollbar');

var _Scrollbar2 = _interopRequireDefault(_Scrollbar);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var handleClass = { add: _domLib.addClass, remove: _domLib.removeClass };
var ReactChildren = _react2.default.Children;

function getTotalByColumns(columns) {
  var totalFlexGrow = 0;
  var totalWidth = 0;

  var count = function count(items) {
    Array.from(items).forEach(function (column) {
      if (_react2.default.isValidElement(column)) {
        var _column$props = column.props,
            flexGrow = _column$props.flexGrow,
            _column$props$width = _column$props.width,
            width = _column$props$width === undefined ? 0 : _column$props$width;

        totalFlexGrow += flexGrow || 0;
        totalWidth += flexGrow ? 0 : width;
      } else if (_lodash2.default.isArray(column)) {
        count(column);
      }
    });
  };
  count(columns);
  return {
    totalFlexGrow: totalFlexGrow,
    totalWidth: totalWidth
  };
}

function cloneCell(Cell, props) {
  return _react2.default.cloneElement(Cell, props);
}

var propTypes = {
  width: _propTypes2.default.number,
  data: _propTypes2.default.array,
  height: _propTypes2.default.number,
  rowHeight: _propTypes2.default.number,
  headerHeight: _propTypes2.default.number,
  onRowClick: _propTypes2.default.func,
  isTree: _propTypes2.default.bool,
  expand: _propTypes2.default.bool,
  locale: _propTypes2.default.object,
  sortColumn: _propTypes2.default.string,
  sortType: _propTypes2.default.oneOf(['desc', 'asc']),
  /**
   * @callback
   * @params: sortColumn dataKey
   * @params: sortType
   */
  onSortColumn: _propTypes2.default.func,
  onRerenderRowHeight: _propTypes2.default.func,
  onTreeToggleOpen: _propTypes2.default.func,
  disabledScroll: _propTypes2.default.bool,
  hover: _propTypes2.default.bool,
  loading: _propTypes2.default.bool,
  onScroll: _propTypes2.default.func
};

var defaultProps = {
  height: 200,
  rowHeight: 36,
  sortType: 'asc',
  hover: true,
  locale: {
    emptyMessage: 'No data found',
    loading: _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement('i', { className: 'icon icon-cog icon-lg icon-spin' }),
      _react2.default.createElement(
        'span',
        null,
        'Loading...'
      )
    )
  }
};

var Table = function (_React$Component) {
  _inherits(Table, _React$Component);

  function Table(props) {
    _classCallCheck(this, Table);

    var _this = _possibleConstructorReturn(this, (Table.__proto__ || Object.getPrototypeOf(Table)).call(this, props));

    _this.state = {
      width: props.width,
      columnWidth: 0,
      dataKey: 0,
      shouldFixedColumn: false,
      contentHeight: 0,
      contentWidth: 0
    };
    _this.treeChildren = {};
    _this.mounted = false;
    _this.reportTableWidth = _this.reportTableWidth.bind(_this);
    _this.onColumnResizeEnd = _this.onColumnResizeEnd.bind(_this);
    _this.onColumnResizeStart = _this.onColumnResizeStart.bind(_this);
    _this.onColumnResizeMove = _this.onColumnResizeMove.bind(_this);
    _this.onTreeToggle = _this.onTreeToggle.bind(_this);
    _this.shouldHandleWheelX = _this.shouldHandleWheelX.bind(_this);
    _this.shouldHandleWheelY = _this.shouldHandleWheelY.bind(_this);
    _this.handleWheel = _this.handleWheel.bind(_this);
    _this.handleScrollX = _this.handleScrollX.bind(_this);
    _this.handleScrollY = _this.handleScrollY.bind(_this);
    return _this;
  }

  _createClass(Table, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      var _this2 = this;

      var _props$children = this.props.children,
          children = _props$children === undefined ? [] : _props$children;

      var shouldFixedColumn = Array.from(children).some(function (child) {
        return child.props && child.props.fixed;
      });

      this.scrollY = 0;
      this.scrollX = 0;
      this.wheelHandler = new _domLib.WheelHandler(function (deltaX, deltaY) {
        _this2.handleWheel(deltaX, deltaY);
        _this2.scrollbarX.onWheelScroll(deltaX);
        _this2.scrollbarY.onWheelScroll(deltaY);
      }, this.shouldHandleWheelX, this.shouldHandleWheelY);
      this.setState({ shouldFixedColumn: shouldFixedColumn });
      this.isMounted = true;
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.onWindowResizeListener = (0, _domLib.on)(window, 'resize', _lodash2.default.debounce(this.reportTableWidth, 400));
      this.reportTableWidth();
      this.reportTableContextHeight();
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      return !(0, _shallowEqual2.default)(this.props, nextProps) || !(0, _shallowEqual2.default)(this.state, nextState);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.reportTableContextHeight();
      this.reportTableContentWidth();
      this.updatePosition();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.onWheelListener) {
        this.onWheelListener.off();
      }
      if (this.onWindowResizeListener) {
        this.onWindowResizeListener.off();
      }
      this.isMounted = false;
    }
  }, {
    key: 'onColumnResizeEnd',
    value: function onColumnResizeEnd(columnWidth, cursorDelta, dataKey, index) {
      this.setState(_defineProperty({
        isColumnResizing: false
      }, dataKey + '_' + index + '_width', columnWidth));
      (0, _domLib.addStyle)(this.mouseArea, {
        display: 'none'
      });
    }
  }, {
    key: 'onColumnResizeStart',
    value: function onColumnResizeStart(width, left, fixed) {
      this.setState({
        isColumnResizing: true
      });
      var mouseAreaLeft = width + left;
      var x = fixed ? mouseAreaLeft : mouseAreaLeft + (this.scrollX || 0);
      (0, _domLib.addStyle)(this.mouseArea, {
        display: 'block',
        transform: 'translate3d(' + x + 'px, 0px, 0px)'
      });
    }
  }, {
    key: 'onColumnResizeMove',
    value: function onColumnResizeMove(width, left, fixed) {
      var mouseAreaLeft = width + left;
      var x = fixed ? mouseAreaLeft : mouseAreaLeft + (this.scrollX || 0);
      (0, _domLib.addStyle)(this.mouseArea, {
        transform: 'translate3d(' + x + 'px, 0px, 0px)'
      });
    }
  }, {
    key: 'onTreeToggle',
    value: function onTreeToggle(rowKey, rowIndex, rowData) {
      var onTreeToggleOpen = this.props.onTreeToggleOpen;

      var expandIcon = (0, _reactDom.findDOMNode)(this.treeChildren[rowKey][rowIndex]);
      var isOpen = (0, _domLib.hasClass)(expandIcon, 'open');

      if (isOpen) {
        (0, _domLib.removeClass)(expandIcon, 'open');
      } else {
        (0, _domLib.addClass)(expandIcon, 'open');
      }

      this.reportTableContextHeight();
      onTreeToggleOpen && onTreeToggleOpen(!isOpen, rowData);
    }
  }, {
    key: 'getScrollCellGroups',
    value: function getScrollCellGroups() {
      var table = (0, _reactDom.findDOMNode)(this.table);
      return table.querySelectorAll('.' + this.prefix('cell-group.scroll'));
    }
  }, {
    key: 'getFixedCellGroups',
    value: function getFixedCellGroups() {
      var table = (0, _reactDom.findDOMNode)(this.table);
      return table.querySelectorAll('.' + this.prefix('cell-group.fixed'));
    }
  }, {
    key: 'getCells',
    value: function getCells() {
      var _this3 = this;

      var left = 0; // Cell left margin
      var headerCells = []; // Table header cell
      var bodyCells = []; // Table body cell
      var columns = this.props.children;

      if (!columns) {
        return {
          headerCells: headerCells,
          bodyCells: bodyCells,
          allColumnsWidth: left
        };
      }

      var tableWidth = this.state.width;
      var _props = this.props,
          sortColumn = _props.sortColumn,
          sortType = _props.sortType,
          onSortColumn = _props.onSortColumn,
          rowHeight = _props.rowHeight,
          headerHeight = _props.headerHeight;

      var _getTotalByColumns = getTotalByColumns(columns),
          totalFlexGrow = _getTotalByColumns.totalFlexGrow,
          totalWidth = _getTotalByColumns.totalWidth;

      ReactChildren.forEach(columns, function (column, index) {

        if (_react2.default.isValidElement(column)) {

          var columnChildren = column.props.children;
          var _column$props2 = column.props,
              width = _column$props2.width,
              fixed = _column$props2.fixed,
              align = _column$props2.align,
              sortable = _column$props2.sortable,
              resizable = _column$props2.resizable,
              flexGrow = _column$props2.flexGrow,
              minWidth = _column$props2.minWidth;


          if (columnChildren.length !== 2) {
            throw new Error('Component <HeaderCell> and <Cell> is required, column index: ' + index + ' ');
          }

          var nextWidth = _this3.state[columnChildren[1].props.dataKey + '_' + index + '_width'] || width || 0;

          if (tableWidth && flexGrow && totalFlexGrow) {
            nextWidth = Math.max((tableWidth - totalWidth) / totalFlexGrow * flexGrow, minWidth || 60);
          }

          var cellProps = {
            fixed: fixed,
            left: left,
            align: align,
            resizable: resizable,
            sortable: sortable,
            index: index,
            headerHeight: headerHeight,
            width: nextWidth,
            height: rowHeight,
            firstColumn: index === 0,
            lastColumn: index === columns.length - 1,
            flexGrow: flexGrow,
            key: index
          };

          var headerCellsProps = {
            headerHeight: headerHeight || rowHeight,
            dataKey: columnChildren[1].props.dataKey,
            sortColumn: sortColumn,
            sortType: sortType,
            onSortColumn: onSortColumn,
            flexGrow: flexGrow
          };

          if (resizable) {
            headerCellsProps.onColumnResizeEnd = _this3.onColumnResizeEnd;
            headerCellsProps.onColumnResizeStart = _this3.onColumnResizeStart;
            headerCellsProps.onColumnResizeMove = _this3.onColumnResizeMove;
          }

          headerCells.push(cloneCell(columnChildren[0], _extends({}, cellProps, headerCellsProps)));
          bodyCells.push(cloneCell(columnChildren[1], cellProps));

          left += nextWidth;
        }
      });

      return {
        headerCells: headerCells,
        bodyCells: bodyCells,
        allColumnsWidth: left
      };
    }
  }, {
    key: 'handleScrollX',
    value: function handleScrollX(delta) {
      this.handleWheel(delta, 0);
    }
  }, {
    key: 'handleScrollY',
    value: function handleScrollY(delta) {
      this.handleWheel(0, delta);
    }
  }, {
    key: 'handleWheel',
    value: function handleWheel(deltaX, deltaY) {
      var onScroll = this.props.onScroll;


      if (!this.isMounted) {
        return;
      }

      var nextScrollX = this.scrollX - deltaX;
      var nextScrollY = this.scrollY - deltaY;

      this.scrollY = Math.min(0, nextScrollY < this.minScrollY ? this.minScrollY : nextScrollY);
      this.scrollX = Math.min(0, nextScrollX < this.minScrollX ? this.minScrollX : nextScrollX);
      this.updatePosition();

      onScroll && onScroll(this.scrollX, this.scrollY);
    }
  }, {
    key: 'updatePosition',
    value: function updatePosition() {
      /**
      * 当存在锁定列情况处理
      */
      if (this.state.shouldFixedColumn) {
        this.updatePositionByFixedCell();
      } else {

        var wheelStyle = {};
        var headerStyle = {};
        (0, _domLib.translateDOMPositionXY)(wheelStyle, this.scrollX, this.scrollY);
        (0, _domLib.translateDOMPositionXY)(headerStyle, this.scrollX, 0);

        this.wheelWrapper && (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
        this.headerWrapper && (0, _domLib.addStyle)(this.headerWrapper, headerStyle);
      }
      handleClass[this.scrollY < 0 ? 'add' : 'remove']((0, _reactDom.findDOMNode)(this.tableHeader), 'shadow');
    }
  }, {
    key: 'updatePositionByFixedCell',
    value: function updatePositionByFixedCell() {
      var _this4 = this;

      var wheelGroupStyle = {};
      var wheelStyle = {};
      var scrollGroups = this.getScrollCellGroups();
      var fixedGroups = this.getFixedCellGroups();

      (0, _domLib.translateDOMPositionXY)(wheelGroupStyle, this.scrollX, 0);
      (0, _domLib.translateDOMPositionXY)(wheelStyle, 0, this.scrollY);

      Array.from(scrollGroups).forEach(function (group) {
        (0, _domLib.addStyle)(group, wheelGroupStyle);
      });

      (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
      Array.from(fixedGroups).forEach(function (group) {
        handleClass[_this4.scrollX < 0 ? 'add' : 'remove'](group, 'shadow');
      });
    }
  }, {
    key: 'shouldHandleWheelX',
    value: function shouldHandleWheelX(delta) {
      var _props2 = this.props,
          disabledScroll = _props2.disabledScroll,
          loading = _props2.loading;

      if (delta === 0 || disabledScroll || loading) {
        return false;
      }

      return delta >= 0 && this.scrollX > this.minScrollX || delta < 0 && this.scrollX < 0;
    }
  }, {
    key: 'shouldHandleWheelY',
    value: function shouldHandleWheelY(delta) {
      var _props3 = this.props,
          disabledScroll = _props3.disabledScroll,
          loading = _props3.loading;

      if (delta === 0 || disabledScroll || loading) {
        return false;
      }
      return delta >= 0 && this.scrollY > this.minScrollY || delta < 0 && this.scrollY < 0;
    }
  }, {
    key: 'randerRowData',
    value: function randerRowData(bodyCells, rowData, props) {
      var _this5 = this;

      var onRowClick = this.props.onRowClick;

      var hasChildren = this.props.isTree && rowData.children && Array.isArray(rowData.children) && rowData.children.length > 0;

      var rowKey = '_' + (Math.random() * 1E18).toString(36).slice(0, 5).toUpperCase();
      var row = this.renderRow({
        key: props.index,
        width: props.rowWidth,
        height: props.rowHeight,
        top: props.top,
        onClick: function onClick() {
          onRowClick && onRowClick(rowData);
        }
      }, bodyCells.map(function (cell) {
        return _react2.default.cloneElement(cell, {
          hasChildren: hasChildren,
          layer: props.layer,
          height: props.rowHeight,
          rowIndex: props.index,
          onTreeToggle: _this5.onTreeToggle,
          rowKey: rowKey,
          rowData: rowData
        });
      }));

      // insert children
      if (hasChildren) {
        props.layer += 1;

        var childrenClasses = (0, _classnames2.default)(this.prefix('row-children'), {
          open: this.props.expand
        });

        return _react2.default.createElement(
          'div',
          {
            className: childrenClasses,
            key: props.index,
            'data-layer': props.layer,
            ref: function ref(_ref) {
              if (!_this5.treeChildren[rowKey]) {
                _this5.treeChildren[rowKey] = {};
              }
              _this5.treeChildren[rowKey][props.index] = _ref;
            }
          },
          row,
          _react2.default.createElement(
            'div',
            { className: 'children' },
            rowData.children.map(function (child, index) {
              return _this5.randerRowData(bodyCells, child, _extends({}, props, {
                index: index
              }));
            })
          )
        );
      }

      return row;
    }
  }, {
    key: 'reportTableWidth',
    value: function reportTableWidth() {
      var table = (0, _reactDom.findDOMNode)(this.table);
      if (table) {
        this.setState({
          width: (0, _domLib.getWidth)(table)
        });
      }
    }
  }, {
    key: 'reportTableContentWidth',
    value: function reportTableContentWidth() {
      var table = (0, _reactDom.findDOMNode)(this.table);

      var row = table.querySelectorAll('.' + this.prefix('row-header'))[0];
      var contentWidth = (0, _domLib.getWidth)(row);

      this.setState({ contentWidth: contentWidth });
      // 这里 -10 是为了让滚动条不挡住内容部分
      this.minScrollX = -(contentWidth - this.state.width) - 10;

      if (this.state.contentWidth !== contentWidth) {
        this.scrollX = 0;
        this.scrollbarX && this.scrollbarX.resetScrollBarPosition();
      }
    }
  }, {
    key: 'reportTableContextHeight',
    value: function reportTableContextHeight() {
      var table = (0, _reactDom.findDOMNode)(this.table);
      var rows = table.querySelectorAll('.' + this.prefix('row'));
      var _props4 = this.props,
          height = _props4.height,
          rowHeight = _props4.rowHeight,
          headerHeight = _props4.headerHeight;

      var contentHeight = 0;
      Array.from(rows).forEach(function (row) {
        contentHeight += (0, _domLib.getHeight)(row);
      });

      var nextContentHeight = contentHeight - (headerHeight || rowHeight);
      this.setState({
        contentHeight: nextContentHeight
      });
      // 这里 -10 是为了让滚动条不挡住内容部分
      this.minScrollY = -(contentHeight - height) - 10;
      if (this.state.contentHeight !== nextContentHeight) {
        this.scrollY = 0;
        this.scrollbarY && this.scrollbarY.resetScrollBarPosition();
      }
    }
  }, {
    key: 'renderRow',
    value: function renderRow(props, cells) {

      // IF there are fixed columns, add a fixed group
      if (this.state.shouldFixedColumn) {

        var fixedCells = cells.filter(function (cell) {
          return cell.props.fixed;
        });
        var otherCells = cells.filter(function (cell) {
          return !cell.props.fixed;
        });
        var fixedCellGroupWidth = 0;

        fixedCells.forEach(function (item) {
          fixedCellGroupWidth += item.props.width;
        });

        return _react2.default.createElement(
          _Row2.default,
          props,
          _react2.default.createElement(
            _CellGroup2.default,
            {
              fixed: true,
              height: props.isHeaderRow ? props.headerHeight : props.height,
              width: fixedCellGroupWidth
            },
            fixedCells
          ),
          _react2.default.createElement(
            _CellGroup2.default,
            null,
            otherCells
          )
        );
      }

      return _react2.default.createElement(
        _Row2.default,
        props,
        _react2.default.createElement(
          _CellGroup2.default,
          null,
          cells
        )
      );
    }
  }, {
    key: 'renderMouseArea',
    value: function renderMouseArea() {
      var _this6 = this;

      var height = this.props.height;

      var styles = { height: height };

      return _react2.default.createElement('div', {
        ref: function ref(_ref2) {
          _this6.mouseArea = _ref2;
        },
        className: this.prefix('mouse-area'),
        style: styles
      });
    }
  }, {
    key: 'renderTableHeader',
    value: function renderTableHeader(headerCells, rowWidth) {
      var _this7 = this;

      var _props5 = this.props,
          rowHeight = _props5.rowHeight,
          headerHeight = _props5.headerHeight;

      var row = this.renderRow({
        ref: function ref(_ref3) {
          _this7.tableHeader = _ref3;
        },
        width: rowWidth,
        height: rowHeight,
        headerHeight: headerHeight,
        isHeaderRow: true,
        top: 0
      }, headerCells);

      return _react2.default.createElement(
        'div',
        {
          ref: function ref(_ref4) {
            _this7.headerWrapper = _ref4;
          },
          className: this.prefix('header-row-wrapper')
        },
        row
      );
    }
  }, {
    key: 'renderTableBody',
    value: function renderTableBody(bodyCells, rowWidth) {
      var _this8 = this;

      var _props6 = this.props,
          headerHeight = _props6.headerHeight,
          rowHeight = _props6.rowHeight,
          height = _props6.height,
          data = _props6.data,
          isTree = _props6.isTree,
          onRerenderRowHeight = _props6.onRerenderRowHeight;


      var bodyStyles = {
        top: isTree ? 0 : headerHeight || rowHeight,
        height: height - (headerHeight || rowHeight)
      };

      var top = 0; // Row position
      var rows = null;
      if (data && data.length > 0) {
        rows = data.map(function (rowData, index) {
          var nextRowHeight = rowHeight;
          /**
           * 自定义行高
           */
          if (onRerenderRowHeight) {
            nextRowHeight = onRerenderRowHeight(rowData) || rowHeight;
          }

          var row = _this8.randerRowData(bodyCells, rowData, {
            index: index,
            top: top,
            rowWidth: rowWidth,
            layer: 0,
            rowHeight: nextRowHeight
          });

          !isTree && (top += nextRowHeight);
          return row;
        });
      }

      var wheelStyles = {
        position: 'absolute'
      };

      return _react2.default.createElement(
        'div',
        {
          ref: function ref(_ref6) {
            _this8.tableBody = _ref6;
          },
          className: this.prefix('body-row-wrapper'),
          style: bodyStyles,
          onWheel: this.wheelHandler.onWheel
        },
        _react2.default.createElement(
          'div',
          {
            style: wheelStyles,
            className: this.prefix('body-wheel-area'),
            ref: function ref(_ref5) {
              _this8.wheelWrapper = _ref5;
            }
          },
          rows
        ),
        this.renderInfo(rows === null),
        this.renderScrollbar(),
        this.renderLoading()
      );
    }
  }, {
    key: 'renderInfo',
    value: function renderInfo(shouldShow) {

      if (!shouldShow) {
        return null;
      }

      var locale = this.props.locale;


      return _react2.default.createElement(
        'div',
        { className: this.prefix('body-info') },
        locale.emptyMessage
      );
    }
  }, {
    key: 'renderScrollbar',
    value: function renderScrollbar() {
      var _this9 = this;

      var _props7 = this.props,
          disabledScroll = _props7.disabledScroll,
          headerHeight = _props7.headerHeight,
          rowHeight = _props7.rowHeight,
          height = _props7.height,
          loading = _props7.loading;
      var _state = this.state,
          contentWidth = _state.contentWidth,
          contentHeight = _state.contentHeight;


      if (disabledScroll || loading) {
        return null;
      }

      return _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(_Scrollbar2.default, {
          length: this.state.width,
          onScroll: this.handleScrollX,
          scrollLength: contentWidth,
          ref: function ref(_ref7) {
            _this9.scrollbarX = _ref7;
          }
        }),
        _react2.default.createElement(_Scrollbar2.default, {
          vertical: true,
          length: height - (headerHeight || rowHeight),
          scrollLength: contentHeight,
          onScroll: this.handleScrollY,
          ref: function ref(_ref8) {
            _this9.scrollbarY = _ref8;
          }
        })
      );
    }

    /**
     *  show loading
     */

  }, {
    key: 'renderLoading',
    value: function renderLoading() {
      var _props8 = this.props,
          loading = _props8.loading,
          locale = _props8.locale;


      if (!loading) {
        return null;
      }

      return _react2.default.createElement(
        'div',
        { className: this.prefix('loading-wrapper') },
        _react2.default.createElement(
          'div',
          { className: this.prefix('loading') },
          locale.loading
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _classNames,
          _this10 = this;

      var _props9 = this.props,
          children = _props9.children,
          className = _props9.className,
          _props9$width = _props9.width,
          width = _props9$width === undefined ? 0 : _props9$width,
          height = _props9.height,
          style = _props9.style,
          rowHeight = _props9.rowHeight,
          isTree = _props9.isTree,
          hover = _props9.hover,
          props = _objectWithoutProperties(_props9, ['children', 'className', 'width', 'height', 'style', 'rowHeight', 'isTree', 'hover']);

      var _getCells = this.getCells(),
          headerCells = _getCells.headerCells,
          bodyCells = _getCells.bodyCells,
          allColumnsWidth = _getCells.allColumnsWidth;

      var rowWidth = allColumnsWidth > width ? allColumnsWidth : width;
      var clesses = (0, _classnames2.default)(_decorate.globalClassName, (_classNames = {}, _defineProperty(_classNames, this.prefix('treetable'), isTree), _defineProperty(_classNames, 'column-resizing', this.state.isColumnResizing), _defineProperty(_classNames, 'table-hover', hover), _classNames), className);

      var styles = _extends({
        width: width || 'auto',
        height: height
      }, style);
      var elementProps = _lodash2.default.omit(props, Object.keys(propTypes));

      return _react2.default.createElement(
        'div',
        _extends({}, elementProps, {
          className: clesses,
          style: styles,
          ref: function ref(_ref9) {
            _this10.table = _ref9;
          }
        }),
        this.renderTableHeader(headerCells, rowWidth),
        children && this.renderTableBody(bodyCells, rowWidth),
        this.renderMouseArea()
      );
    }
  }, {
    key: 'isMounted',
    get: function get() {
      return this.mounted;
    },
    set: function set(isMounted) {
      this.mounted = isMounted;
    }
  }]);

  return Table;
}(_react2.default.Component);

Table.propTypes = propTypes;
Table.defaultProps = defaultProps;

exports.default = (0, _decorate2.default)()(Table);